@@ -25,6 +25,7 @@ import pyghmi.constants as const
 import pyghmi.exceptions as exc
 import pyghmi.ipmi.events as sel
 import pyghmi.ipmi.fru as fru
+import pyghmi.ipmi.oem.generic as genericoem
 from pyghmi.ipmi.oem.lookup import get_oem_handler
 import pyghmi.ipmi.private.util as pygutil
 from pyghmi.ipmi import sdr
@@ -249,6 +250,10 @@ class Command(object):
         """
         if self._oemknown:
             return
+        if self.bmc is None:
+            self._oem = genericoem.OEMHandler(None, None)
+            self._oemknown = True
+            return
         self._oem, self._oemknown = get_oem_handler(self._get_device_id(),
                                                     self)
 
@@ -269,12 +274,12 @@ class Command(object):
             raise exc.IpmiException(response['error'])
         # this should only be invoked for get system boot option complying to
         # ipmi spec and targeting the 'boot flags' parameter
-        assert (response['command'] == 9 and
-                response['netfn'] == 1 and
-                response['data'][0] == 1 and
-                (response['data'][1] & 0b1111111) == 5)
-        if (response['data'][1] & 0b10000000 or
-                not response['data'][2] & 0b10000000):
+        assert (response['command'] == 9
+                and response['netfn'] == 1
+                and response['data'][0] == 1
+                and (response['data'][1] & 0b1111111) == 5)
+        if (response['data'][1] & 0b10000000
+                or not response['data'][2] & 0b10000000):
             return {'bootdev': 'default', 'persistent': True}
         else:  # will consult data2 of the boot flags parameter for the data
             persistent = False
@@ -345,8 +350,7 @@ class Command(object):
         waitattempts = 300
         if not isinstance(wait, bool):
             waitattempts = wait
-        if (wait and
-                newpowerstate in ('on', 'off', 'shutdown', 'softoff')):
+        if wait and newpowerstate in ('on', 'off', 'shutdown', 'softoff'):
             if newpowerstate in ('softoff', 'shutdown'):
                 waitpowerstate = 'off'
             else:
@@ -1029,7 +1033,7 @@ class Command(object):
                         netchan = bytearray(rsp['data'])[0]
                         self._netchannel = netchan & 0b1111
                         break
-                    except exc.IpmiException as ie:
+                    except exc.IpmiException:
                         # This means the attempt to fetch parameter 5 failed,
                         # therefore move on to next candidate channel
                         continue
@@ -1216,8 +1220,9 @@ class Command(object):
                                                          channel)
             if destdata:
                 self.xraw_command(netfn=0xc, command=1, data=destdata)
-        if (acknowledge_required is not None or retries is not None or
-                acknowledge_timeout is not None):
+        if (acknowledge_required is not None
+                or retries is not None
+                or acknowledge_timeout is not None):
             currtype = self.xraw_command(netfn=0xc, command=2, data=(
                 channel, 18, destination, 0))
             if currtype['data'][0] != b'\x11':
@@ -1758,15 +1763,16 @@ class Command(object):
         }
         data = [uid, mode_mask[mode]]
         if password:
-            password = str(password)
+            if not isinstance(password, bytes):
+                password = password.encode('utf8')
             if 21 > len(password) > 16:
                 password = password.ljust(20, b'\x00')
                 data[0] |= 0b10000000
             elif len(password) > 20:
                 raise Exception('password has limit of 20 chars')
             else:
-                password = password.ljust(16, "\x00")
-            data.extend([ord(x) for x in password])
+                password = password.ljust(16, b'\x00')
+            data.extend(bytearray(password))
         try:
             self.xraw_command(netfn=0x06, command=0x47, data=data)
         except exc.IpmiException as ie:
