@@ -1,6 +1,5 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2014 IBM Corporation
+# Copyright 2015-2019 Lenovo
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,12 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# This represents the low layer message framing portion of IPMI
+"""This represents the low layer message framing portion of IPMI"""
 
-import pyghmi.exceptions as exc
 import struct
 import threading
 
+import pyghmi.exceptions as exc
 from pyghmi.ipmi.private import constants
 from pyghmi.ipmi.private import session
 from pyghmi.ipmi.private.util import _monotonic_time
@@ -74,8 +73,8 @@ class Console(object):
             self.callgotsession = None
 
     def _got_session(self, response):
-        """Private function to navigate SOL payload activation
-        """
+        """Private function to navigate SOL payload activation"""
+
         if 'error' in response:
             self._print_error(response['error'])
             return
@@ -182,22 +181,22 @@ class Console(object):
             if isinstance(data, dict):
                 self.pendingoutput.append(data)
             else:  # it is a text situation
-                if (len(self.pendingoutput) == 0 or
-                        isinstance(self.pendingoutput[-1], dict)):
+                if (len(self.pendingoutput) == 0
+                        or isinstance(self.pendingoutput[-1], dict)):
                     self.pendingoutput.append(data)
                 else:
                     self.pendingoutput[-1] += data
 
     def _got_cons_input(self, handle):
-        """Callback for handle events detected by ipmi session
-        """
+        """Callback for handle events detected by ipmi session"""
+
         self._addpendingdata(handle.read())
         if not self.awaitingack:
             self._sendpendingoutput()
 
     def close(self):
-        """Shut down an SOL session,
-        """
+        """Shut down an SOL session"""
+
         if self.ipmi_session:
             self.ipmi_session.unregister_keepalive(self.keepaliveid)
         if self.activated:
@@ -237,23 +236,25 @@ class Console(object):
 
     def _sendpendingoutput(self):
         with self.outputlock:
+            dobreak = False
+            chunk = ''
             if len(self.pendingoutput) == 0:
                 return
             if isinstance(self.pendingoutput[0], dict):
                 if 'break' in self.pendingoutput[0]:
-                    self._sendoutput("", sendbreak=True)
+                    dobreak = True
                 else:
+                    del self.pendingoutput[0]
                     raise ValueError
                 del self.pendingoutput[0]
-                return
-            if len(self.pendingoutput[0]) > self.maxoutcount:
+            elif len(self.pendingoutput[0]) > self.maxoutcount:
                 chunk = self.pendingoutput[0][:self.maxoutcount]
                 self.pendingoutput[0] = self.pendingoutput[0][
                     self.maxoutcount:]
             else:
                 chunk = self.pendingoutput[0]
                 del self.pendingoutput[0]
-            self._sendoutput(chunk)
+            self._sendoutput(chunk, sendbreak=dobreak)
 
     def _sendoutput(self, output, sendbreak=False):
         self.myseq += 1
@@ -309,8 +310,8 @@ class Console(object):
         self.broken = True
         if self.ipmi_session:
             self.ipmi_session.unregister_keepalive(self.keepaliveid)
-            if (self.ipmi_session.sol_handler and
-                    self.ipmi_session.sol_handler.__self__ is self):
+            if (self.ipmi_session.sol_handler
+                    and self.ipmi_session.sol_handler.__self__ is self):
                 self.ipmi_session.sol_handler = None
             self.ipmi_session = None
         if type(error) == dict:
@@ -328,8 +329,8 @@ class Console(object):
         self.out_handler(data)
 
     def _got_sol_payload(self, payload):
-        """SOL payload callback
-        """
+        """SOL payload callback"""
+
         # TODO(jbjohnso) test cases to throw some likely scenarios at functions
         # for example, retry with new data, retry with no new data
         # retry with unexpected sequence number
@@ -387,8 +388,9 @@ class Console(object):
                     # also add pending output for efficiency and ease
                     newtext = self.lastpayload[4 + ackcount:]
                     with self.outputlock:
-                        if (self.pendingoutput and
-                                not isinstance(self.pendingoutput[0], dict)):
+                        if (self.pendingoutput
+                                and not isinstance(self.pendingoutput[0],
+                                                   dict)):
                             self.pendingoutput[0] = \
                                 newtext + self.pendingoutput[0]
                         else:
@@ -454,8 +456,8 @@ class ServerConsole(Console):
         session.Session.wait_for_rsp(0)
 
     def _got_sol_payload(self, payload):
-        """SOL payload callback
-        """
+        """SOL payload callback"""
+
         # TODO(jbjohnso) test cases to throw some likely scenarios at functions
         # for example, retry with new data, retry with no new data
         # retry with unexpected sequence number
@@ -509,8 +511,8 @@ class ServerConsole(Console):
             if nacked and not breakdetected:  # the BMC was in some way unhappy
                 newtext = self.lastpayload[4 + ackcount:]
                 with self.outputlock:
-                    if (self.pendingoutput and
-                            not isinstance(self.pendingoutput[0], dict)):
+                    if (self.pendingoutput
+                            and not isinstance(self.pendingoutput[0], dict)):
                         self.pendingoutput[0] = newtext + self.pendingoutput[0]
                     else:
                         self.pendingoutput = [newtext] + self.pendingoutput
@@ -537,6 +539,6 @@ class ServerConsole(Console):
                                        needskeepalive=needskeepalive)
 
     def close(self):
-        """Shut down an SOL session,
-        """
+        """Shut down an SOL session"""
+
         self.activated = False
