@@ -311,10 +311,10 @@ def _fix_sel_time(records, ipmicmd):
         record = records[index]
         if 'timecode' not in record or record['timecode'] == 0xffffffff:
             continue
-        if ('event' in record and record['event'] == 'Clock time change' and
-                record['event_data'] == 'After'):
-            if (lasttimestamp is not None and
-                    record['timecode'] > lasttimestamp):
+        if ('event' in record and record['event'] == 'Clock time change'
+                and record['event_data'] == 'After'):
+            if (lasttimestamp is not None
+                    and record['timecode'] > lasttimestamp):
                 # if the timestamp did something impossible, declare the rest
                 # of history not meaningfully correctable
                 correctionenabled = False
@@ -322,8 +322,8 @@ def _fix_sel_time(records, ipmicmd):
                 continue
             newtimestamp = record['timecode']
             trimindexes.append(index)
-        elif ('event' in record and record['event'] == 'Clock time change' and
-                record['event_data'] == 'Before'):
+        elif ('event' in record and record['event'] == 'Clock time change'
+              and record['event_data'] == 'Before'):
             if not correctionenabled:
                 continue
             if newtimestamp:
@@ -342,8 +342,8 @@ def _fix_sel_time(records, ipmicmd):
                 if not correctearly or not correctionenabled:
                     correctednowtime = nowtime
                     continue
-                if (lasttimestamp is not None and
-                        record['timecode'] > lasttimestamp):
+                if (lasttimestamp is not None
+                        and record['timecode'] > lasttimestamp):
                     # Time has gone backwards in pre-init, no hope for
                     # accurate time
                     correctearly = False
@@ -359,8 +359,8 @@ def _fix_sel_time(records, ipmicmd):
                 # We are in 'normal' time, assume we cannot go to
                 # pre-init time and do corrections unless time sync events
                 # guide us in safely
-                if (lasttimestamp is not None and
-                        record['timecode'] > lasttimestamp):
+                if (lasttimestamp is not None
+                        and record['timecode'] > lasttimestamp):
                     # Time has gone backwards, without a clock sync
                     # give up any attempt to correct from this point back...
                     correctionenabled = False
@@ -474,7 +474,7 @@ class EventHandler(object):
         # evtoffset = specifictrap[3] & 0b1111
         deassertion = (specifictrap[3] & 0b10000000) == 0b10000000
         # alertseverity = petdata[26]
-        sensorid = petdata[28]
+        sensorid = '{0}.0'.format(petdata[28])
         event_data = petdata[31:34]
         event = {}
         seqnum = struct.unpack_from('>H', buffer(petdata[16:18]))[0]
@@ -498,7 +498,8 @@ class EventHandler(object):
             raise pygexc.PyghmiException(
                 'Unrecognized Event message version {0}'.format(eventdata[2]))
         sensor_type = eventdata[3]
-        sensorid = eventdata[4]
+        sensorid = '{0}.{1}.{2}'.format(
+            eventdata[0], eventdata[4], eventdata[1] & 0b11)
         event_data = eventdata[6:]
         deassertion = (eventdata[5] & 0b10000000 == 0b10000000)
         event_type = eventdata[5] & 0b1111111
