@@ -277,9 +277,9 @@ class OEMHandler(generic.OEMHandler):
                 event['severity'] = pygconst.Health.Critical
         # For HDD bay events, the event data 2 is the bay, modify
         # the description to be more specific
-        if (event['event_type_byte'] == 0x6f and
-                (evdata[0] & 0b11000000) == 0b10000000 and
-                event['component_type_id'] == 13):
+        if (event['event_type_byte'] == 0x6f
+                and (evdata[0] & 0b11000000) == 0b10000000
+                and event['component_type_id'] == 13):
             event['component'] += ' {0}'.format(evdata[1] & 0b11111)
 
     def reseat_bay(self, bay):
@@ -366,6 +366,7 @@ class OEMHandler(generic.OEMHandler):
                                       self.oemid['product_id'])
 
     tsma_ids = ((19046, 32, 1287),)
+
     @property
     def has_tsma(self):
         currid = (self.oemid['manufacturer_id'], self.oemid['device_id'],
@@ -376,8 +377,8 @@ class OEMHandler(generic.OEMHandler):
     def has_tsm(self):
         """True if this particular server have a TSM based service processor"""
 
-        if (self.oemid['manufacturer_id'] == 19046 and
-                self.oemid['device_id'] == 32):
+        if (self.oemid['manufacturer_id'] == 19046
+                and self.oemid['device_id'] == 32):
             try:
                 self.ipmicmd.xraw_command(netfn=0x3a, command=0xf)
             except pygexc.IpmiException as ie:
@@ -985,7 +986,8 @@ class OEMHandler(generic.OEMHandler):
         if self.has_xcc:
             return self.immhandler.set_bmc_configuration(changeset)
         if self.is_fpc:
-            return self.smmhandler.set_bmc_configuration(changeset)
+            return self.smmhandler.set_bmc_configuration(
+                changeset, self._fpc_variant)
         return super(OEMHandler, self).set_bmc_configuration(changeset)
 
     def get_system_configuration(self, hideadvanced):
