@@ -27,6 +27,8 @@ import socket
 import struct
 import weakref
 
+import six
+
 import pyghmi.constants as pygconst
 import pyghmi.exceptions as pygexc
 import pyghmi.ipmi.oem.lenovo.config as config
@@ -37,7 +39,6 @@ from pyghmi.ipmi import sdr
 import pyghmi.media as media
 import pyghmi.storage as storage
 import pyghmi.util.webclient as webclient
-import six
 
 try:
     from urllib import urlencode
@@ -123,8 +124,11 @@ class IMMClient(object):
         self.ipmicmd = weakref.proxy(ipmicmd)
         self.updating = False
         self.imm = ipmicmd.bmc
+        srv = self.imm
+        if ':' in srv:
+            srv = '[{0}]'.format(srv)
         self.adp_referer = 'https://{0}/designs/imm/index-console.php'.format(
-            self.imm)
+            srv)
         if ipmicmd.ipmi_session.password:
             self.username = ipmicmd.ipmi_session.userid.decode('utf-8')
             self.password = ipmicmd.ipmi_session.password.decode('utf-8')
@@ -188,8 +192,8 @@ class IMMClient(object):
         try:
             self.fwo = self.fwc.get_fw_options(fetchimm=fetchimm)
         except Exception:
-            raise Exception(self.bmcname +
-                            ' failed to retrieve UEFI configuration')
+            raise Exception('%s failed to retrieve UEFI configuration'
+                            % self.bmcname)
         self.fwovintage = util._monotonic_time()
         retcfg = {}
         for opt in self.fwo:
@@ -197,8 +201,8 @@ class IMMClient(object):
                 # Suppress the Avago configuration to be consistent with
                 # other tools.
                 continue
-            if (hideadvanced and self.fwo[opt]['lenovo_protect'] or
-                    self.fwo[opt]['hidden']):
+            if (hideadvanced and self.fwo[opt]['lenovo_protect']
+                    or self.fwo[opt]['hidden']):
                 # Do not enumerate hidden settings
                 continue
             retcfg[opt] = {}
@@ -266,8 +270,8 @@ class IMMClient(object):
             newnewvalues = []
             for newvalue in newvalues:
                 newv = re.sub(r'\s+', ' ', newvalue)
-                if (self.fwo[key]['possible'] and
-                        newvalue not in self.fwo[key]['possible']):
+                if (self.fwo[key]['possible']
+                        and newvalue not in self.fwo[key]['possible']):
                     candlist = []
                     for candidate in self.fwo[key]['possible']:
                         candid = re.sub(r'\s+', ' ', candidate)
@@ -331,6 +335,8 @@ class IMMClient(object):
             if not isinstance(ret, str):
                 ret = ret.decode('utf-8')
             return ret
+        if propdata[0] == 0x44:  # dword
+            return propdata[1:5]
         else:
             raise Exception('Unknown format for property: ' + repr(propdata))
 
@@ -370,8 +376,13 @@ class IMMClient(object):
 
     @property
     def wc(self):
-        if (not self._wc or (self._wc.vintage and
-                             self._wc.vintage < util._monotonic_time() - 30)):
+        if (not self._wc or (self._wc.vintage
+                             and self._wc.vintage < util._monotonic_time()
+                             - 30)):
+            if not self.updating and self._wc:
+                # in case the existing session is still valid
+                # dispose of the session
+                self.weblogout()
             self._wc = self.get_webclient()
         return self._wc
 
@@ -548,22 +559,22 @@ class IMMClient(object):
                         bdata = {}
                         if 'versionStr' in firm and firm['versionStr']:
                             bdata['version'] = firm['versionStr']
-                        if ('releaseDate' in firm and
-                                firm['releaseDate'] and
-                                firm['releaseDate'] != 'N/A'):
+                        if ('releaseDate' in firm
+                                and firm['releaseDate']
+                                and firm['releaseDate'] != 'N/A'):
                             try:
                                 bdata['date'] = self._parse_builddate(
                                     firm['releaseDate'])
                             except ValueError:
                                 pass
-                        yield ('{0} {1}'.format(aname, fname), bdata)
+                        yield '{0} {1}'.format(aname, fname), bdata
                 for fwi in fwu.get('items', []):
                     if fwi.get('key', -1) == adata.get('key', -2):
                         if fwi.get('fw_status', 0) == 2:
                             bdata = {}
                             if 'fw_version_pend' in fwi:
                                 bdata['version'] = fwi['fw_version_pend']
-                            yield('{0} Pending Update'.format(aname), bdata)
+                            yield '{0} Pending Update'.format(aname), bdata
         for disk in self.disk_inventory():
             yield disk
         self.weblogout()
@@ -704,8 +715,8 @@ class IMMClient(object):
                     bdata['pcislot'] = '{0:02x}:{1:02x}'.format(
                         fundata[self.BUSNO], fundata[self.DEVNO])
                     serialdata = fundata.get(self.ADP_SERIALNO, None)
-                    if (serialdata and serialdata != 'N/A' and
-                            '---' not in serialdata):
+                    if (serialdata and serialdata != 'N/A'
+                            and '---' not in serialdata):
                         bdata['serial'] = serialdata
                     partnum = fundata.get(self.ADP_PARTNUM, None)
                     if partnum and partnum != 'N/A':
@@ -886,6 +897,13 @@ class XCCClient(IMMClient):
             settings['smm']['value'] = 'Enable'
         else:
             settings['smm']['value'] = None
+        smmip = self.get_property('/v2/ibmc/smm/smm_ip')
+        if smmip:
+            smmip = socket.inet_ntoa(bytes(smmip[-1::-1]))
+            settings['smm_ip'] = {
+                'help': 'Current IPv4 address as reported by SMM, read-only',
+                'value': smmip,
+            }
         return settings
 
     rulemap = {
@@ -925,9 +943,8 @@ class XCCClient(IMMClient):
             '/redfish/v1/Systems/1/Bios/Actions/Bios.ResetBios',
             {'Action': 'Bios.ResetBios'},
             headers={
-                'Authorization': 'Basic ' +
-                                 base64.b64encode(
-                                     self.username + ':' + self.password),
+                'Authorization': 'Basic %s' % base64.b64encode(
+                    self.username + ':' + self.password),
                 'Content-Type': 'application/json'
             }
         )
@@ -952,12 +969,16 @@ class XCCClient(IMMClient):
                             'password': self.password
                             })
         headers = {'Connection': 'keep-alive',
+                   'Referer': 'https://xcc/',
+                   'Host': 'xcc',
                    'Content-Type': 'application/json'}
         wc.request('POST', '/api/login', adata, headers)
         rsp = wc.getresponse()
         if rsp.status == 200:
             rspdata = json.loads(rsp.read())
             wc.set_header('Content-Type', 'application/json')
+            wc.set_header('Referer', 'https://xcc/')
+            wc.set_header('Host', 'xcc')
             wc.set_header('Authorization', 'Bearer ' + rspdata['access_token'])
             if '_csrf_token' in wc.cookies:
                 wc.set_header('X-XSRF-TOKEN', wc.cookies['_csrf_token'])
@@ -965,9 +986,18 @@ class XCCClient(IMMClient):
 
     def _raid_number_map(self, controller):
         themap = {}
+        cid = controller.split(',')
         rsp = self.wc.grab_json_response(
             '/api/function/raid_conf?'
-            'params=raidlink_GetDisksToConf,{0}'.format(controller))
+            'params=raidlink_GetDisksToConf,{0}'.format(cid[0]))
+        if rsp.get('return') == 22:  # New style firmware
+            if cid[2] == 2:
+                arg = '{0},{1}'.format(cid[1], cid[2])
+            else:
+                arg = '{0},{1}'.format(cid[0], cid[2])
+            arg = 'params=raidlink_GetDisksToConf,{0}'.format(arg)
+            rsp = self.wc.grab_json_response(
+                '/api/function/raid_conf?{0}'.format(arg))
         for lvl in rsp['items'][0]['supported_raidlvl']:
             mapdata = (lvl['rdlvl'], lvl['maxSpan'])
             raidname = lvl['rdlvlstr'].replace(' ', '').lower()
@@ -1105,11 +1135,18 @@ class XCCClient(IMMClient):
             else:
                 hstr = ''
             drvstr = '|'.join([str(x.id[1]) for x in drives]) + '|'
+            ctl = controller.split(',')
             pth = '/api/function/raid_conf?params=raidlink_CheckConfisValid'
-            args = [pth, controller, rdlvl, spancount, drivesperspan, drvstr,
+            args = [pth, ctl[0], rdlvl, spancount, drivesperspan, drvstr,
                     hstr]
             url = ','.join([str(x) for x in args])
             rsp = self.wc.grab_json_response(url)
+            if rsp.get('return', -1) == 22:
+                args.append(ctl[1])
+                args = [pth, ctl[0], rdlvl, spancount, drivesperspan, drvstr,
+                        hstr, ctl[1]]
+                url = ','.join([str(x) for x in args])
+                rsp = self.wc.grab_json_response(url)
             if rsp['items'][0]['errcode'] == 16:
                 raise pygexc.InvalidParameterValue('Incorrect number of disks')
             elif rsp['items'][0]['errcode'] != 0:
@@ -1184,9 +1221,18 @@ class XCCClient(IMMClient):
         realcfg = self.get_storage_configuration(False)
         for pool in cfgspec.arrays:
             for volume in pool.volumes:
-                vid = '{0},{1}'.format(volume.id[1], volume.id[0])
+                cid = volume.id[0].split(',')
+                if cid[2] == 2:
+                    vid = '{0},{1},{2}'.format(volume.id[1], cid[1], cid[2])
+                else:
+                    vid = '{0},{1},{2}'.format(volume.id[1], cid[0], cid[2])
                 rsp = self.wc.grab_json_response(
                     '/api/function', {'raidlink_RemoveVolumeAsync': vid})
+                if rsp.get('return', -1) == 2:
+                    # older firmware
+                    vid = '{0},{1}'.format(volume.id[1], cid[0])
+                    rsp = self.wc.grab_json_response(
+                        '/api/function', {'raidlink_RemoveVolumeAsync': vid})
                 if rsp.get('return', -1) != 0:
                     raise Exception(
                         'Unexpected return to volume deletion: ' + repr(rsp))
@@ -1212,8 +1258,9 @@ class XCCClient(IMMClient):
 
     def _create_array(self, pool):
         params = self._parse_array_spec(pool)
+        cid = params['controller'].split(',')[0]
         url = '/api/function/raid_conf?params=raidlink_GetDefaultVolProp'
-        args = (url, params['controller'], 0, params['drives'])
+        args = (url, cid, 0, params['drives'])
         props = self.wc.grab_json_response(','.join([str(x) for x in args]))
         props = props['items'][0]
         volumes = pool.volumes
@@ -1248,8 +1295,8 @@ class XCCClient(IMMClient):
             elif strsize in ('remainder', 'rest'):
                 volsize = remainingcap
             elif strsize.endswith('%'):
-                volsize = int(params['capacity'] *
-                              float(strsize.replace('%', '')) / 100.0)
+                volsize = int(params['capacity']
+                              * float(strsize.replace('%', '')) / 100.0)
             else:
                 try:
                     volsize = int(strsize)
@@ -1264,12 +1311,23 @@ class XCCClient(IMMClient):
                 name, volsize, stripsize, props['cpwb'], props['cpra'],
                 props['cpio'], props['ap'], props['dcp'], props['initstate']))
         url = '/api/function'
+        cid = params['controller'].split(',')
+        cnum = cid[0]
         arglist = '{0},{1},{2},{3},{4},{5},'.format(
-            params['controller'], params['raidlevel'], params['spans'],
+            cnum, params['raidlevel'], params['spans'],
             params['perspan'], params['drives'], params['hotspares'])
         arglist += ''.join(vols)
         parms = {'raidlink_AddNewVolWithNaAsync': arglist}
         rsp = self.wc.grab_json_response(url, parms)
+        if rsp['return'] == 14:  # newer firmware
+            if cid[2] == 2:
+                cnum = cid[1]
+            arglist = '{0},{1},{2},{3},{4},{5},'.format(
+                cnum, params['raidlevel'], params['spans'],
+                params['perspan'], params['drives'], params['hotspares'])
+            arglist += ''.join(vols) + ',{0}'.format(cid[2])
+            parms = {'raidlink_AddNewVolWithNaAsync': arglist}
+            rsp = self.wc.grab_json_response(url, parms)
         if rsp['return'] != 0:
             raise Exception(
                 'Unexpected response to add volume command: ' + repr(rsp))
@@ -1330,12 +1388,17 @@ class XCCClient(IMMClient):
 
     def get_storage_configuration(self, logout=True):
         rsp = self.wc.grab_json_response(
-            '/api/function/raid_alldevices?params=storage_GetAllDevices')
+            '/api/function/raid_alldevices?params=storage_GetAllDevices,0')
+        if not rsp:
+            rsp = self.wc.grab_json_response(
+                '/api/function/raid_alldevices?params=storage_GetAllDevices')
         standalonedisks = []
         pools = []
         for item in rsp.get('items', []):
             for cinfo in item['controllerInfo']:
-                cid = cinfo['id']
+                cid = '{0},{1},{2}'.format(
+                    cinfo['id'], cinfo.get('slotNo', -1), cinfo.get(
+                        'type', -1))
                 for pool in cinfo['pools']:
                     volumes = []
                     disks = []
@@ -1441,8 +1504,8 @@ class XCCClient(IMMClient):
         # then check for agentless, if agentless, get adapter info using
         # https, using the caller TLS verification scheme
         components = set(components)
-        if (not components or
-                set(('core', 'imm', 'bmc', 'xcc')) & components):
+        if (not components
+                or set(('core', 'imm', 'bmc', 'xcc')) & components):
             rsp = self.ipmicmd.xraw_command(netfn=0x3a, command=0x50)
             immverdata = self.parse_imm_buildinfo(rsp['data'])
             bmcmajor, bmcminor = [int(x) for x in bmcver.split('.')]
@@ -1450,20 +1513,20 @@ class XCCClient(IMMClient):
             bdata = {'version': bmcver,
                      'build': immverdata[0],
                      'date': immverdata[1]}
-            yield (self.bmcname, bdata)
+            yield self.bmcname, bdata
             bdata = self.fetch_grouped_properties({
                 'build': '/v2/ibmc/dm/fw/imm3/backup_pending_build_id',
                 'version': '/v2/ibmc/dm/fw/imm3/backup_pending_build_version',
                 'date': '/v2/ibmc/dm/fw/imm3/backup_pending_build_date'})
             if bdata:
-                yield ('{0} Backup'.format(self.bmcname), bdata)
+                yield '{0} Backup'.format(self.bmcname), bdata
             else:
                 bdata = self.fetch_grouped_properties({
                     'build': '/v2/ibmc/dm/fw/imm3/backup_build_id',
                     'version': '/v2/ibmc/dm/fw/imm3/backup_build_version',
                     'date': '/v2/ibmc/dm/fw/imm3/backup_build_date'})
                 if bdata:
-                    yield ('{0} Backup'.format(self.bmcname), bdata)
+                    yield '{0} Backup'.format(self.bmcname), bdata
                     bdata = self.fetch_grouped_properties({
                         'build': '/v2/ibmc/trusted_buildid',
                     })
@@ -1472,7 +1535,7 @@ class XCCClient(IMMClient):
                     'build': '/v2/ibmc/trusted_buildid',
                 })
             if bdata:
-                yield ('{0} Trusted Image'.format(self.bmcname), bdata)
+                yield '{0} Trusted Image'.format(self.bmcname), bdata
             bdata = self.fetch_grouped_properties({
                 'build': '/v2/ibmc/dm/fw/imm3/primary_pending_build_id',
                 'version': '/v2/ibmc/dm/fw/imm3/primary_pending_build_version',
@@ -1562,8 +1625,8 @@ class XCCClient(IMMClient):
                 url = mt['remotepath']
                 if url.startswith('//'):
                     url = 'smb:' + url
-                elif (not url.startswith('http://') and
-                      not url.startswith('https://')):
+                elif (not url.startswith('http://')
+                      and not url.startswith('https://')):
                     url = url.replace(':', '')
                     url = 'nfs://' + url
                 yield media.Media(mt['filename'], url)
@@ -1603,10 +1666,14 @@ class XCCClient(IMMClient):
         if rsp.get('return', -1) != 0:
             errmsg = repr(rsp) if rsp else self.wc.lastjsonerror
             raise Exception('Unrecognized return: ' + errmsg)
-        rsp = self.wc.grab_json_response('/api/providers/rp_rdoc_getfiles')
-        if 'items' not in rsp or len(rsp['items']) == 0:
-            raise Exception(
-                'Image upload was not accepted, it may be too large')
+        ready = False
+        while not ready:
+            ipmisession.Session.pause(3)
+            rsp = self.wc.grab_json_response('/api/providers/rp_rdoc_getfiles')
+            if 'items' not in rsp or len(rsp['items']) == 0:
+                raise Exception(
+                    'Image upload was not accepted, it may be too large')
+            ready = rsp['items'][0]['size'] != 0
         self._refresh_token()
         rsp = self.wc.grab_json_response('/api/providers/rp_rdoc_mountall',
                                          {})
@@ -1680,8 +1747,8 @@ class XCCClient(IMMClient):
                 progress({'phase': 'upload',
                           'progress': 100.0 * rsp['received'] / rsp['size']})
             elif rsp['state'] != 'done':
-                if (rsp.get('status', None) == 413 or
-                    uploadthread.rspstatus == 413):
+                if (rsp.get('status', None) == 413
+                        or uploadthread.rspstatus == 413):
                     raise Exception('File is larger than supported')
                 raise Exception('Unexpected result:' + repr(rsp))
             uploadstate = rsp['state']
@@ -1810,8 +1877,7 @@ class XCCClient(IMMClient):
 
         if rsp.get('return', -1) != 0:
             errmsg = repr(rsp) if rsp else self.wc.lastjsonerror
-            raise Exception('Unexpected result starting update: ' +
-                            errmsg)
+            raise Exception('Unexpected result starting update: %s' % errmsg)
         complete = False
         while not complete:
             ipmisession.Session.pause(3)
@@ -1827,8 +1893,8 @@ class XCCClient(IMMClient):
                 if rsp['items'][0]['action_status'] != 0:
                     raise Exception('Unexpected failure: ' + repr(rsp))
                 break
-            if (rsp['items'][0]['action_state'] == 'In Progress' and
-                    rsp['items'][0]['action_status'] == 2):
+            if (rsp['items'][0]['action_state'] == 'In Progress'
+                    and rsp['items'][0]['action_status'] == 2):
                 raise Exception('Unexpected failure: ' + repr(rsp))
             if rsp['items'][0]['action_state'] != 'In Progress':
                 raise Exception(
@@ -1866,7 +1932,7 @@ class XCCClient(IMMClient):
     def get_health(self, summary):
         try:
             wc = self.get_webclient(False)
-        except (socket.timeout, socket.error) as e:
+        except (socket.timeout, socket.error):
             wc = None
         if not wc:
             summary['health'] = pygconst.Health.Critical
@@ -1895,12 +1961,16 @@ class XCCClient(IMMClient):
             'E': pygconst.Health.Critical,
             'W': pygconst.Health.Warning,
         }
+        infoevents = False
         for item in rsp.get('items', ()):
             # while usually the ipmi interrogation shall explain things,
             # just in case there is a gap, make sure at least the
             # health field is accurately updated
             itemseverity = hmap.get(item.get('severity', 'E'),
                                     pygconst.Health.Critical)
+            if itemseverity == pygconst.Health.Ok:
+                infoevents = True
+                continue
             if (summary['health'] < itemseverity):
                 summary['health'] = itemseverity
             if item['cmnid'] == 'FQXSPPW0104J':
@@ -1928,7 +1998,8 @@ class XCCClient(IMMClient):
                     'health': itemseverity,
                     'type': item['source'],
                 }, ''))
-        if summary.get('health', pygconst.Health.Ok) == pygconst.Health.Ok:
+        if (summary.get('health', pygconst.Health.Ok) == pygconst.Health.Ok
+                and not infoevents):
             # Fault LED is lit without explanation, mark to encourage
             # examination
             summary['health'] = pygconst.Health.Warning
