@@ -30,6 +30,7 @@ from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.ciphers import algorithms
 from cryptography.hazmat.primitives.ciphers import Cipher
 from cryptography.hazmat.primitives.ciphers import modes
+
 import pyghmi.exceptions as exc
 from pyghmi.ipmi.private import constants
 from pyghmi.ipmi.private import util
@@ -206,8 +207,8 @@ def _io_graball(mysockets, iowaiters):
             myport = mysocket.getsockname()[1]
             rdata = rdata + (mysocket,)
             relsession = None
-            if (rdata[1] in Session.bmc_handlers and
-                    myport in Session.bmc_handlers[rdata[1]]):
+            if (rdata[1] in Session.bmc_handlers
+                    and myport in Session.bmc_handlers[rdata[1]]):
                 # session data
                 rdata = rdata + (True,)
                 relsession = Session.bmc_handlers[rdata[1]][myport]
@@ -436,13 +437,15 @@ class Session(object):
             if sockaddr in cls.bmc_handlers:
                 for portself in list(dictitems(cls.bmc_handlers[sockaddr])):
                     self = portself[1]
-                    if not ((self.logged or self.logging) and
-                            cls._is_session_valid(self)):
+                    if not ((self.logged or self.logging)
+                            and cls._is_session_valid(self)):
                         # we have encountered a leftover broken session
                         del cls.bmc_handlers[sockaddr][portself[0]]
                         continue
-                    if (self.bmc == bmc and self.userid == userid and
-                            self.password == password and self.kgo == kg):
+                    if (self.bmc == bmc
+                            and self.userid == userid
+                            and self.password == password
+                            and self.kgo == kg):
                         trueself = self
                         break
                     # ok, the candidate seems to be working, but does not match
@@ -670,9 +673,10 @@ class Session(object):
         # we are always the requestor for now
         seqincrement = 7  # IPMI spec forbids gaps bigger then 7 in seq number.
         # Risk the taboo rather than violate the rules
-        while (not self.servermode and
-                (netfn, command, self.seqlun) in self.tabooseq and
-               self.tabooseq[(netfn, command, self.seqlun)] and seqincrement):
+        while (not self.servermode
+               and (netfn, command, self.seqlun) in self.tabooseq
+               and self.tabooseq[(netfn, command, self.seqlun)]
+               and seqincrement):
             self.tabooseq[(self.expectednetfn, command, self.seqlun)] -= 1
             # Allow taboo to eventually expire after a few rounds
             self.seqlun += 1  # the last two bits are lun, so add 4 to add 1
@@ -747,8 +751,8 @@ class Session(object):
                 waiter({'success': True})
             self.process_pktqueue()
             with util.protect(WAITING_SESSIONS):
-                if (self in self.waiting_sessions and
-                        self.expiration < _monotonic_time()):
+                if (self in self.waiting_sessions
+                        and self.expiration < _monotonic_time()):
                     self.waiting_sessions.pop(self, None)
                     self._timedout()
 
@@ -763,8 +767,8 @@ class Session(object):
                     callback=None,
                     rslun=0):
         if not self.logged:
-            if (self.logoutexpiry is not None and
-                    _monotonic_time() > self.logoutexpiry):
+            if (self.logoutexpiry is not None
+                    and _monotonic_time() > self.logoutexpiry):
                 self._mark_broken()
             raise exc.IpmiException('Session no longer connected')
         self._cmdwait()
@@ -928,8 +932,8 @@ class Session(object):
         # advance idle timer since we don't need keepalive while sending
         # packets out naturally
         with util.protect(KEEPALIVE_SESSIONS):
-            if (self in Session.keepalive_sessions and not needskeepalive and
-                    not self._customkeepalives):
+            if (self in Session.keepalive_sessions and not needskeepalive
+                    and not self._customkeepalives):
                 Session.keepalive_sessions[self]['timeout'] = \
                     _monotonic_time() + MAX_IDLE - (random.random() * 4.9)
             self._xmit_packet(retry, delay_xmit=delay_xmit, timeout=timeout)
@@ -1149,8 +1153,8 @@ class Session(object):
                     if parms['timeout'] <= curtime:
                         timeout = 0  # exit after one guaranteed pass
                         break
-                    if (timeout is not None and
-                            timeout < parms['timeout'] - curtime):
+                    if (timeout is not None
+                            and timeout < parms['timeout'] - curtime):
                         continue  # timeout smaller than the current session
                         # needs
                     timeout = parms['timeout'] - curtime  # set new timeout
@@ -1160,8 +1164,8 @@ class Session(object):
                     if parms['timeout'] <= curtime:
                         timeout = 0
                         break
-                    if (timeout is not None and
-                            timeout < parms['timeout'] - curtime):
+                    if (timeout is not None
+                            and timeout < parms['timeout'] - curtime):
                         continue
                     timeout = parms['timeout'] - curtime
         # If the loop above found no sessions wanting *and* the caller had no
@@ -1305,17 +1309,17 @@ class Session(object):
     def _handle_ipmi_packet(self, data, sockaddr=None, qent=None):
         if self.sockaddr is None and sockaddr is not None:
             self.sockaddr = sockaddr
-        elif (self.sockaddr is not None and
-              sockaddr is not None and
-              self.sockaddr != sockaddr):
+        elif (self.sockaddr is not None
+              and sockaddr is not None
+              and self.sockaddr != sockaddr):
             return  # here, we might have sent an ipv4 and ipv6 packet to kick
             # things off ignore the second reply since we have one
             # satisfactory answer
         if data[4] in (0, 2):  # This is an ipmi 1.5 paylod
             remseqnumber = struct.unpack('<I', bytes(data[5:9]))[0]
             remsessid = struct.unpack("<I", bytes(data[9:13]))[0]
-            if (remseqnumber == 0 and remsessid == 0 and
-                    qent[2] in Session.bmc_handlers):
+            if (remseqnumber == 0 and remsessid == 0
+                    and qent[2] in Session.bmc_handlers):
                 # So a new ipmi client happens to get a previously seen and
                 # still active UDP source port.  Clear ourselves out and punt
                 # to IpmiServer
@@ -1324,8 +1328,8 @@ class Session(object):
                 iserver.pktqueue.append(qent)
                 iserver.process_pktqueue()
                 return
-            if (hasattr(self, 'remseqnumber') and
-                    remseqnumber < self.remseqnumber):
+            if (hasattr(self, 'remseqnumber')
+                    and remseqnumber < self.remseqnumber):
                 return -5  # remote sequence number is too low, reject it
             self.remseqnumber = remseqnumber
             if data[4] != self.authtype:
@@ -1402,9 +1406,9 @@ class Session(object):
             if sid != self.localsid:  # session id mismatch, drop it
                 return
             remseqnumber = struct.unpack("<I", bytes(data[10:14]))[0]
-            if (hasattr(self, 'remseqnumber') and
-                (remseqnumber < self.remseqnumber) and
-                    (self.remseqnumber != 0xffffffff)):
+            if (hasattr(self, 'remseqnumber')
+                    and (remseqnumber < self.remseqnumber)
+                    and (self.remseqnumber != 0xffffffff)):
                 return
             self.remseqnumber = remseqnumber
             psize = data[14] + (data[15] << 8)
@@ -1520,11 +1524,11 @@ class Session(object):
         self.remoterandombytes = bytes(data[8:24])
         self.remoteguid = bytes(data[24:40])
         userlen = len(self.userid)
-        hmacdata = (struct.pack("<II", localsid, self.pendingsessionid) +
-                    self.randombytes + self.remoterandombytes +
-                    self.remoteguid + struct.pack(
-                    "2B", self.nameonly | self.privlevel, userlen) +
-                    self.userid)
+        hmacdata = (struct.pack("<II", localsid, self.pendingsessionid)
+                    + self.randombytes + self.remoterandombytes
+                    + self.remoteguid + struct.pack(
+                    "2B", self.nameonly | self.privlevel, userlen)
+                    + self.userid)
         expectedhash = hmac.new(self.password, hmacdata,
                                 self.currhashlib).digest()
         hashlen = len(expectedhash)
@@ -1536,10 +1540,10 @@ class Session(object):
         # We have now validated that the BMC and client agree on password, time
         # to store the keys
         self.sik = hmac.new(self.kg,
-                            self.randombytes + self.remoterandombytes +
-                            struct.pack("2B", self.nameonly | self.privlevel,
-                                        userlen) +
-                            self.userid, self.currhashlib).digest()
+                            self.randombytes + self.remoterandombytes
+                            + struct.pack(
+                                "2B", self.nameonly | self.privlevel, userlen)
+                            + self.userid, self.currhashlib).digest()
         self.k1 = hmac.new(self.sik, b'\x01' * 20, self.currhashlib).digest()
         self.k2 = hmac.new(self.sik, b'\x02' * 20, self.currhashlib).digest()
         self.aeskey = self.k2[0:16]
@@ -1640,8 +1644,8 @@ class Session(object):
             # second response is the real response.
             # If the message is send crrectly, we will discard the first
             # response or else error message will be parsed and return.
-            if ((entry[0] in [0x06, 0x07]) and (entry[2] == 0x34) and
-                    (payload[-2] == 0x0)):
+            if ((entry[0] in [0x06, 0x07]) and (entry[2] == 0x34)
+                    and (payload[-2] == 0x0)):
                 return -1
             else:
                 self._parse_payload(payload)
@@ -1718,8 +1722,8 @@ class Session(object):
             # delayed in this case, it's safe to just redo the request
             self.lastpayload = None
             self._open_rmcpplus_request()
-        elif (self.sessioncontext == 'EXPECTINGRAKP2' or
-              self.sessioncontext == 'EXPECTINGRAKP4'):
+        elif (self.sessioncontext == 'EXPECTINGRAKP2'
+              or self.sessioncontext == 'EXPECTINGRAKP4'):
             # If we can't be sure which RAKP was dropped or if RAKP3/4 was just
             # delayed, the most reliable thing to do is rewind and start over
             # bmcs do not take kindly to receiving RAKP1 or RAKP3 twice
@@ -1827,8 +1831,8 @@ class Session(object):
             # replace this one
             myport = self.socket.getsockname()[1]
             for sockaddr in self.allsockaddrs:
-                if (sockaddr in Session.bmc_handlers and
-                        myport in Session.bmc_handlers[sockaddr]):
+                if (sockaddr in Session.bmc_handlers
+                        and myport in Session.bmc_handlers[sockaddr]):
                     del Session.bmc_handlers[sockaddr][myport]
                     if Session.bmc_handlers[sockaddr] == {}:
                         del Session.bmc_handlers[sockaddr]
