@@ -12,19 +12,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import pyghmi.redfish.oem.generic as generic
-from pyghmi.util.parse import parse_time
+
 import errno
 import json
 import math
 import os
+import random
 import socket
 import time
-import pyghmi.ipmi.private.util as util
+
 import pyghmi.exceptions as pygexc
+import pyghmi.ipmi.private.util as util
+import pyghmi.redfish.oem.generic as generic
 import pyghmi.storage as storage
+from pyghmi.util.parse import parse_time
 import pyghmi.util.webclient as webclient
-import random
 
 
 class OEMHandler(generic.OEMHandler):
@@ -50,7 +52,8 @@ class OEMHandler(generic.OEMHandler):
         return {'height': u_height, 'slot': slot}
 
     def _get_agentless_firmware(self, components):
-        adata = self.wc.grab_json_response('/api/dataset/imm_adapters?params=pci_GetAdapters')
+        adata = self.wc.grab_json_response(
+            '/api/dataset/imm_adapters?params=pci_GetAdapters')
         anames = set()
         for adata in adata.get('items', []):
             baseaname = adata['adapterName']
@@ -75,14 +78,14 @@ class OEMHandler(generic.OEMHandler):
                     bdata = {}
                     if 'versionStr' in firm and firm['versionStr']:
                         bdata['version'] = firm['versionStr']
-                    if ('releaseDate' in firm and
-                            firm['releaseDate'] and
-                            firm['releaseDate'] != 'N/A'):
+                    if ('releaseDate' in firm
+                            and firm['releaseDate']
+                            and firm['releaseDate'] != 'N/A'):
                         try:
                             bdata['date'] = parse_time(firm['releaseDate'])
                         except ValueError:
                             pass
-                    yield ('{0} {1}'.format(aname, fname), bdata)
+                    yield '{0} {1}'.format(aname, fname), bdata
 
     def _get_disk_firmware_single(self, diskent, prefix=''):
         bdata = {}
@@ -93,6 +96,7 @@ class OEMHandler(generic.OEMHandler):
             'productName'].rstrip()
         bdata['version'] = diskent['fwVersion']
         return (diskname, bdata)
+
     def _get_disk_firmware(self, coponents):
         storagedata = storagedata = self.wc.grab_json_response(
             '/api/function/raid_alldevices?params=storage_GetAllDisks')
@@ -131,12 +135,17 @@ class OEMHandler(generic.OEMHandler):
 
     def get_storage_configuration(self, logout=True):
         rsp = self.wc.grab_json_response(
-            '/api/function/raid_alldevices?params=storage_GetAllDevices')
+            '/api/function/raid_alldevices?params=storage_GetAllDevices,0')
+        if not rsp:
+            rsp = self.wc.grab_json_response(
+                '/api/function/raid_alldevices?params=storage_GetAllDevices')
         standalonedisks = []
         pools = []
         for item in rsp.get('items', []):
             for cinfo in item['controllerInfo']:
-                cid = cinfo['id']
+                cid = '{0},{1},{2}'.format(
+                    cinfo['id'], cinfo.get('slotNo', -1), cinfo.get(
+                        'type', -1))
                 for pool in cinfo['pools']:
                     volumes = []
                     disks = []
@@ -228,9 +237,18 @@ class OEMHandler(generic.OEMHandler):
 
     def _raid_number_map(self, controller):
         themap = {}
+        cid = controller.split(',')
         rsp = self.wc.grab_json_response(
             '/api/function/raid_conf?'
-            'params=raidlink_GetDisksToConf,{0}'.format(controller))
+            'params=raidlink_GetDisksToConf,{0}'.format(cid[0]))
+        if rsp.get('return') == 22:  # New style firmware
+            if cid[2] == 2:
+                arg = '{0},{1}'.format(cid[1], cid[2])
+            else:
+                arg = '{0},{1}'.format(cid[0], cid[2])
+            arg = 'params=raidlink_GetDisksToConf,{0}'.format(arg)
+            rsp = self.wc.grab_json_response(
+                '/api/function/raid_conf?{0}'.format(arg))
         for lvl in rsp['items'][0]['supported_raidlvl']:
             mapdata = (lvl['rdlvl'], lvl['maxSpan'])
             raidname = lvl['rdlvlstr'].replace(' ', '').lower()
@@ -279,11 +297,18 @@ class OEMHandler(generic.OEMHandler):
             else:
                 hstr = ''
             drvstr = '|'.join([str(x.id[1]) for x in drives]) + '|'
+            ctl = controller.split(',')
             pth = '/api/function/raid_conf?params=raidlink_CheckConfisValid'
-            args = [pth, controller, rdlvl, spancount, drivesperspan, drvstr,
+            args = [pth, ctl[0], rdlvl, spancount, drivesperspan, drvstr,
                     hstr]
             url = ','.join([str(x) for x in args])
             rsp = self.wc.grab_json_response(url)
+            if rsp.get('return', -1) == 22:
+                args.append(ctl[1])
+                args = [pth, ctl[0], rdlvl, spancount, drivesperspan, drvstr,
+                        hstr, ctl[1]]
+                url = ','.join([str(x) for x in args])
+                rsp = self.wc.grab_json_response(url)
             if rsp['items'][0]['errcode'] == 16:
                 raise pygexc.InvalidParameterValue('Incorrect number of disks')
             elif rsp['items'][0]['errcode'] != 0:
@@ -300,12 +325,15 @@ class OEMHandler(generic.OEMHandler):
                 'perspan': drivesperspan,
             }
         else:
-            pass  # TODO: adding new volume to existing array would be here
+            # TODO(Jarrod Johnson): adding new volume to
+            #  existing array would be here
+            pass
 
     def _create_array(self, pool):
         params = self._parse_array_spec(pool)
+        cid = params['controller'].split(',')[0]
         url = '/api/function/raid_conf?params=raidlink_GetDefaultVolProp'
-        args = (url, params['controller'], 0, params['drives'])
+        args = (url, cid, 0, params['drives'])
         props = self.wc.grab_json_response(','.join([str(x) for x in args]))
         props = props['items'][0]
         volumes = pool.volumes
@@ -340,8 +368,9 @@ class OEMHandler(generic.OEMHandler):
             elif strsize in ('remainder', 'rest'):
                 volsize = remainingcap
             elif strsize.endswith('%'):
-                volsize = int(params['capacity'] *
-                              float(strsize.replace('%', '')) / 100.0)
+                volsize = int(params['capacity']
+                              * float(strsize.replace('%', ''))
+                              / 100.0)
             else:
                 try:
                     volsize = int(strsize)
@@ -356,12 +385,23 @@ class OEMHandler(generic.OEMHandler):
                 name, volsize, stripsize, props['cpwb'], props['cpra'],
                 props['cpio'], props['ap'], props['dcp'], props['initstate']))
         url = '/api/function'
+        cid = params['controller'].split(',')
+        cnum = cid[0]
         arglist = '{0},{1},{2},{3},{4},{5},'.format(
-            params['controller'], params['raidlevel'], params['spans'],
+            cnum, params['raidlevel'], params['spans'],
             params['perspan'], params['drives'], params['hotspares'])
         arglist += ''.join(vols)
         parms = {'raidlink_AddNewVolWithNaAsync': arglist}
         rsp = self.wc.grab_json_response(url, parms)
+        if rsp['return'] == 14:  # newer firmware
+            if cid[2] == 2:
+                cnum = cid[1]
+            arglist = '{0},{1},{2},{3},{4},{5},'.format(
+                cnum, params['raidlevel'], params['spans'],
+                params['perspan'], params['drives'], params['hotspares'])
+            arglist += ''.join(vols) + ',{0}'.format(cid[2])
+            parms = {'raidlink_AddNewVolWithNaAsync': arglist}
+            rsp = self.wc.grab_json_response(url, parms)
         if rsp['return'] != 0:
             raise Exception(
                 'Unexpected response to add volume command: ' + repr(rsp))
@@ -371,9 +411,18 @@ class OEMHandler(generic.OEMHandler):
         realcfg = self.get_storage_configuration(False)
         for pool in cfgspec.arrays:
             for volume in pool.volumes:
-                vid = '{0},{1}'.format(volume.id[1], volume.id[0])
+                cid = volume.id[0].split(',')
+                if cid[2] == 2:
+                    vid = '{0},{1},{2}'.format(volume.id[1], cid[1], cid[2])
+                else:
+                    vid = '{0},{1},{2}'.format(volume.id[1], cid[0], cid[2])
                 rsp = self.wc.grab_json_response(
                     '/api/function', {'raidlink_RemoveVolumeAsync': vid})
+                if rsp.get('return', -1) == 2:
+                    # older firmware
+                    vid = '{0},{1}'.format(volume.id[1], cid[0])
+                    rsp = self.wc.grab_json_response(
+                        '/api/function', {'raidlink_RemoveVolumeAsync': vid})
                 if rsp.get('return', -1) != 0:
                     raise Exception(
                         'Unexpected return to volume deletion: ' + repr(rsp))
@@ -397,8 +446,9 @@ class OEMHandler(generic.OEMHandler):
 
     @property
     def wc(self):
-        if (not self._wc or (self._wc.vintage and
-                             self._wc.vintage < util._monotonic_time() - 30)):
+        if (not self._wc or (self._wc.vintage
+                             and self._wc.vintage < util._monotonic_time()
+                             - 30)):
             self._wc = self.get_webclient()
         return self._wc
 
@@ -413,10 +463,13 @@ class OEMHandler(generic.OEMHandler):
             return None
         if not login:
             return wc
+        referer = 'https://xcc/'
         adata = json.dumps({'username': self.username,
                             'password': self.password
                             })
         headers = {'Connection': 'keep-alive',
+                   'Referer': referer,
+                   'Host': 'xcc',
                    'Content-Type': 'application/json'}
         wc.request('POST', '/api/login', adata, headers)
         rsp = wc.getresponse()
@@ -424,6 +477,8 @@ class OEMHandler(generic.OEMHandler):
             rspdata = json.loads(rsp.read())
             wc.set_header('Content-Type', 'application/json')
             wc.set_header('Authorization', 'Bearer ' + rspdata['access_token'])
+            wc.set_header('Referer', referer)
+            wc.set_header('Host', 'xcc')
             if '_csrf_token' in wc.cookies:
                 wc.set_header('X-XSRF-TOKEN', wc.cookies['_csrf_token'])
             return wc
@@ -435,7 +490,8 @@ class OEMHandler(generic.OEMHandler):
             vminfo = self._do_web_request(vmurl, cache=False)
             if vminfo['ConnectedVia'] != 'NotConnected':
                 continue
-            self._do_web_request(vmurl, {'Image': url, 'Inserted': True}, 'PATCH')
+            self._do_web_request(vmurl, {'Image': url, 'Inserted': True},
+                                 'PATCH')
             raise pygexc.BypassGenericBehavior()
             break
         else:
@@ -472,10 +528,14 @@ class OEMHandler(generic.OEMHandler):
         if rsp.get('return', -1) != 0:
             errmsg = repr(rsp) if rsp else self.wc.lastjsonerror
             raise Exception('Unrecognized return: ' + errmsg)
-        rsp = self.wc.grab_json_response('/api/providers/rp_rdoc_getfiles')
-        if 'items' not in rsp or len(rsp['items']) == 0:
-            raise Exception(
-                'Image upload was not accepted, it may be too large')
+        ready = False
+        while not ready:
+            time.sleep(3)
+            rsp = self.wc.grab_json_response('/api/providers/rp_rdoc_getfiles')
+            if 'items' not in rsp or len(rsp['items']) == 0:
+                raise Exception(
+                    'Image upload was not accepted, it may be too large')
+            ready = rsp['items'][0]['size'] != 0
         self._refresh_token()
         rsp = self.wc.grab_json_response('/api/providers/rp_rdoc_mountall',
                                          {})
@@ -485,7 +545,6 @@ class OEMHandler(generic.OEMHandler):
             raise Exception('Unrecognized return: ' + errmsg)
         if progress:
             progress({'phase': 'complete'})
-        #self.weblogout()
 
     def update_firmware(self, filename, data=None, progress=None, bank=None):
         result = None
@@ -501,15 +560,12 @@ class OEMHandler(generic.OEMHandler):
             self._refresh_token()
             self.wc.grab_json_response('/api/providers/fwupdate', json.dumps(
                 {'UPD_WebCancel': 1}))
-            #self.weblogout()
             raise
         self.updating = False
-        #self.weblogout()
         return result
 
     def update_firmware_backend(self, filename, data=None, progress=None,
                                 bank=None):
-        #self.weblogout()
         self._refresh_token()
         rsv = self.wc.grab_json_response('/api/providers/fwupdate', json.dumps(
             {'UPD_WebReserve': 1}))
@@ -530,7 +586,8 @@ class OEMHandler(generic.OEMHandler):
                 progress({'phase': 'upload',
                           'progress': 100.0 * rsp['received'] / rsp['size']})
             elif rsp['state'] != 'done':
-                if rsp.get('status', None) == 413 or uploadthread.rspstatus == 413:
+                if (rsp.get('status', None) == 413
+                        or uploadthread.rspstatus == 413):
                     raise Exception('File is larger than supported')
                 raise Exception('Unexpected result:' + repr(rsp))
             uploadstate = rsp['state']
@@ -565,6 +622,9 @@ class OEMHandler(generic.OEMHandler):
             raise Exception('Update image not intended for this system')
         elif rsp.get('return', -1) == 108:
             raise Exception('Temporary error validating update, try again')
+        elif rsp.get('return', -1) == 109:
+            raise Exception('Invalid update file or component '
+                            'does not support remote update')
         elif rsp.get('return', -1) != 0:
             errmsg = repr(rsp) if rsp else self.wc.lastjsonerror
             raise Exception('Unexpected return to verify: ' + errmsg)
@@ -575,10 +635,13 @@ class OEMHandler(generic.OEMHandler):
             rsp, status = self.wc.grab_json_response_with_status(
                 '/api/providers/fwupdate',
                 json.dumps({'UPD_WebVerifyUploadFileStatus': 1}))
-            if not rsp or status != 200  or rsp.get('return', -1) == 2:
+            if not rsp or status != 200 or rsp.get('return', -1) == 2:
                 # The XCC firmware predates the FileStatus api
                 verifyuploadfilersp = rsp
                 break
+            if rsp.get('return', -1) == 109:
+                raise Exception('Invalid update file or component '
+                                'does not support remote update')
             if rsp.get('return', -1) != 0:
                 errmsg = repr(rsp) if rsp else self.wc.lastjsonerror
                 raise Exception(
@@ -607,7 +670,7 @@ class OEMHandler(generic.OEMHandler):
                 'TDM', 'WINDOWS DRIV', 'LINUX DRIVER', 'UEFI', 'IMM'):
             # adapter firmware
             webid = rsp['items'][0]['webfile_build_id']
-            locations = webid[webid.find('[')+1:webid.find(']')]
+            locations = webid[webid.find('[') + 1:webid.find(']')]
             locations = locations.split(':')
             validselectors = set([])
             for loc in locations:
@@ -632,7 +695,6 @@ class OEMHandler(generic.OEMHandler):
                 errmsg = repr(rsp) if rsp else self.wc.lastjsonerror
                 raise Exception(
                     'Unexpected result from PCI select: ' + errmsg)
-            self.set_property('/v2/ibmc/uefi/force-inventory', 1)
         else:
             self._refresh_token()
             rsp = self.wc.grab_json_response(
@@ -653,8 +715,7 @@ class OEMHandler(generic.OEMHandler):
 
         if rsp.get('return', -1) != 0:
             errmsg = repr(rsp) if rsp else self.wc.lastjsonerror
-            raise Exception('Unexpected result starting update: ' +
-                            errmsg)
+            raise Exception('Unexpected result starting update: %s' % errmsg)
         complete = False
         while not complete:
             time.sleep(3)
@@ -668,20 +729,20 @@ class OEMHandler(generic.OEMHandler):
             if rsp['items'][0]['action_state'] == 'Complete OK':
                 complete = True
                 if rsp['items'][0]['action_status'] != 0:
-                    raise Exception('Unexpected failure: ' + repr(rsp))
+                    raise Exception('Unexpected failure: %s' % repr(rsp))
                 break
-            if (rsp['items'][0]['action_state'] == 'In Progress' and
-                    rsp['items'][0]['action_status'] == 2):
+            if (rsp['items'][0]['action_state'] == 'In Progress'
+                    and rsp['items'][0]['action_status'] == 2):
                 raise Exception('Unexpected failure: ' + repr(rsp))
             if rsp['items'][0]['action_state'] != 'In Progress':
                 raise Exception(
                     'Unknown condition waiting for '
-                    'firmware update: ' + repr(rsp))
+                    'firmware update: %s' % repr(rsp))
         if bank == 'backup':
             return 'complete'
         return 'pending'
 
-    def get_diagnostic_data(self, savefile, progress=None):
+    def get_diagnostic_data(self, savefile, progress=None, autosuffix=False):
         self.wc.grab_json_response('/api/providers/ffdc',
                                    {'Generate_FFDC': 1})
         percent = 0
@@ -697,6 +758,8 @@ class OEMHandler(generic.OEMHandler):
             result = self.wc.grab_json_response('/api/providers/ffdc',
                                                 {'Generate_FFDC_status': 1})
         url = '/ffdc/{0}'.format(result['FileName'])
+        if autosuffix and not savefile.endswith('.tzz'):
+            savefile += '.tzz'
         fd = webclient.FileDownloader(self.wc, url, savefile)
         fd.start()
         while fd.isAlive():
@@ -705,8 +768,11 @@ class OEMHandler(generic.OEMHandler):
                 progress({'phase': 'download',
                           'progress': 100 * self.wc.get_download_progress()})
             self._refresh_token()
+        if fd.exc:
+            raise fd.exc
         if progress:
             progress({'phase': 'complete'})
+        return savefile
 
     def get_licenses(self):
         licdata = self.wc.grab_json_response('/api/providers/imm_fod')
@@ -714,7 +780,10 @@ class OEMHandler(generic.OEMHandler):
             if lic['status'] == 0:
                 yield {'name': lic['feature'], 'state': 'Active'}
             if lic['status'] == 10:
-                yield {'name': lic['feature'], 'state': 'Missing required license'}
+                yield {
+                    'name': lic['feature'],
+                    'state': 'Missing required license'
+                }
 
     def delete_license(self, name):
         licdata = self.wc.grab_json_response('/api/providers/imm_fod')
@@ -722,7 +791,11 @@ class OEMHandler(generic.OEMHandler):
             if lic.get('feature', None) == name:
                 licid = ','.join((str(lic['type']), str(lic['id'])))
                 self.wc.grab_json_response(
-                    '/api/providers/imm_fod', {'FOD_LicenseKeyDelete': licid})
+                    '/api/providers/imm_fod',
+                    {
+                        'FOD_LicenseKeyDelete': licid
+                    }
+                )
                 break
 
     def save_licenses(self, directory):
@@ -756,8 +829,12 @@ class OEMHandler(generic.OEMHandler):
         rsp = json.loads(uploadthread.rsp)
         licpath = rsp.get('items', [{}])[0].get('path', None)
         if licpath:
-            rsp = self.wc.grab_json_response('/api/providers/imm_fod',
-                                             {'FOD_LicenseKeyInstall': licpath})
+            rsp = self.wc.grab_json_response(
+                '/api/providers/imm_fod',
+                {
+                    'FOD_LicenseKeyInstall': licpath
+                }
+            )
             if rsp.get('return', 0) in license_errors:
                 raise pygexc.InvalidParameterValue(
                     license_errors[rsp['return']])
